-- Performance Monitoring
do
	local ProfileMark = 20000

        -- metatable
        local mt = {
                __call = function(self,...)
                        -- Note the beginning time
                        self.begin = GetTick()

                        -- Determine the time between calls
                        if self.end_time ~= nil then
                                self.TicksBetween = self.TicksBetween + (self.begin - self.end_time)
                        end

                        -- Call the function
                        local ret = {self.func(unpack(arg))}

                        -- Get the end time
                        self.end_time = GetTick()

                        -- Update variables
                        local delta = self.end_time - self.begin

                        self.TicksSpent = self.TicksSpent + delta
                        if delta > self.TicksLongest then
                                self.TicksLongest = delta
                        end
                        self.CyclesRun = self.CyclesRun + 1

                        -- Output the data if enough time has passed
                        if self.last_output + ProfileMark < GetTick() then
                                TraceAI(self.level .. string.format (
                                        " -- %s mark (%d ms since last; %d cycles; %.3f ms longest; %.3f ms avg cycle; %.3f ms avg between) -- ",
                                        self.name,
                                        GetTick() - self.last_output,
                                        self.CyclesRun,
                                        self.TicksLongest,
                                        self.TicksSpent / self.CyclesRun,
                                        self.TicksBetween / self.CyclesRun)
                                )
                                if self.include_mem then
                                        TraceAI(self.level .. string.format (
                                                "               --> memory usage: %d kb; gc threshold: %d kb",
                                                gcinfo())
                                        )
                                end

                                self.TicksLongest = 0
                                self.TicksSpent = 0
                                self.TicksBetween = 0
                                self.CyclesRun = 0
                                self.last_output = GetTick()
                        end


                end,
        }

        ProfilingHook = function(n,f,l,inc_mem)
                local ret = {
                        name = n,
                        func = f,
                        level = l,
                        last_output = GetTick(),
                        include_mem = inc_mem,
                        TicksBetween = 0,
                        TicksSpent = 0,
                        TicksLongest = 0,
                        CyclesRun = 0,
                }

                setmetatable(ret,mt)

                return ret
        end
end